@@ -1,24 +1,45 @@
 # Copyright (c) 2011, Found IT A/S and Piped Project Contributors.
 # See LICENSE for details.
 import json
+from StringIO import StringIO
 
 from twisted.application import service
 from twisted.internet import defer, address
 from twisted.python import filepath, failure
 from twisted.trial import unittest
-from twisted.web import resource
+from twisted.web import resource, server, http_headers
+from twisted.web.test import test_web
 
 from piped import exceptions, util, processing, dependencies
 from piped.providers import web_provider
 
 
-class StubPipeline(object):
+class DummyRequest(test_web.DummyRequest, server.Request):
+    channel = Ellipsis
 
-    def __init__(self, processor):
-        self.processor = processor
+    def __init__(self, *a, **kw):
+        test_web.DummyRequest.__init__(self, *a, **kw)
+        self.requestHeaders = http_headers.Headers()
+        self.content = StringIO()
 
-    def process(self, baton):
-        return self.processor(baton)
+    def getHeader(self, key):
+        return server.Request.getHeader(self, key)
+
+    def setHeader(self, name, value):
+        return server.Request.setHeader(self, name, value)
+
+    def set_content(self, content):
+        if not hasattr(content, 'read'):
+            self.content = StringIO(content)
+        else:
+            self.content = content
+
+    def setResponseCode(self, code, message=None):
+        server.Request.setResponseCode(self, code, message)
+
+    @property
+    def written_as_string(self):
+        return ''.join(self.written)
 
 
 class WebProviderTest(unittest.TestCase):
@@ -43,22 +64,22 @@ class WebProviderTest(unittest.TestCase):
         web_resource.configure(self.runtime_environment)
         return web_resource
 
-    def assertConfiguredWithPipeline(self, web_resource, pipeline=None, missing_pipeline=None):
-        if pipeline:
-            self.assertNotEquals(web_resource.pipeline_dependency, None)
-            self.assertEquals(web_resource.pipeline_dependency.provider, pipeline)
+    def assertConfiguredWithProcessor(self, web_resource, processor=None, no_resource_processor=None):
+        if processor:
+            self.assertNotEquals(web_resource.processor_dependency, None)
+            self.assertEquals(web_resource.processor_dependency.provider, processor)
         else:
-            self.assertEquals(web_resource.pipeline_dependency, None)
+            self.assertEquals(web_resource.processor_dependency, None)
 
-        if missing_pipeline:
-            self.assertNotEquals(web_resource.missing_pipeline_dependency, None)
-            self.assertEquals(web_resource.missing_pipeline_dependency.provider, missing_pipeline)
+        if no_resource_processor:
+            self.assertNotEquals(web_resource.no_resource_processor_dependency, None)
+            self.assertEquals(web_resource.no_resource_processor_dependency.provider, no_resource_processor)
         else:
-            self.assertEquals(web_resource.missing_pipeline_dependency, None)
+            self.assertEquals(web_resource.no_resource_processor_dependency, None)
 
     def getResourceForFakeRequest(self, site, post_path=None, request=None):
         if not request:
-            request = web_provider.DummyRequest(post_path)
+            request = DummyRequest(post_path)
         return site.factory.getResourceFor(request)
 
     def getConfiguredWebSite(self, config):
@@ -69,39 +90,39 @@ class WebProviderTest(unittest.TestCase):
     def test_enabled_web_sites_provided(self):
         provider = web_provider.WebResourceProvider()
         self.configuration_manager.set('web.my_site.routing',
-           dict(__config__=dict(pipeline='a_pipeline'))
+           dict(__config__=dict(processor='a_processor'))
         )
         self.configuration_manager.set('web.another_site.enabled', False)
         self.configuration_manager.set('web.another_site.routing',
-            dict(__config__=dict(pipeline='a_pipeline'))
+            dict(__config__=dict(processor='a_processor'))
         )
 
         provider.configure(self.runtime_environment)
 
         self.assertEquals(len(provider.services), 1)
 
-    def test_simple_pipeline_routing(self):
+    def test_simple_processor_routing(self):
         config = dict(
             routing = dict(
-                __config__ = dict(pipeline='a_pipeline')
+                __config__ = dict(processor='pipeline.a_pipeline')
             )
         )
 
         web_site = self.getConfiguredWebSite(config)
 
         web_resource = self.getResourceForFakeRequest(web_site, [''])
-        self.assertConfiguredWithPipeline(web_resource, 'pipeline.a_pipeline')
+        self.assertConfiguredWithProcessor(web_resource, 'pipeline.a_pipeline')
 
-    def test_missing_pipeline_routing(self):
+    def test_no_resource_processor_routing(self):
         config = dict(
             routing = dict(
-                __config__ = dict(pipeline='root_pipeline', missing_pipeline='root_missing_pipeline'),
+                __config__ = dict(processor='pipeline.root_pipeline', no_resource_processor='pipeline.root_no_resource_pipeline'),
                 foo = dict(
-                    __config__ = dict(pipeline = 'foo_pipeline')
+                    __config__ = dict(processor = 'pipeline.foo_pipeline')
                 ),
                 bar = dict(
                     baz = dict(
-                        __config__ = dict(missing_pipeline = 'baz_pipeline')
+                        __config__ = dict(no_resource_processor = 'pipeline.baz_pipeline')
                     )
                 )
             )
@@ -110,41 +131,41 @@ class WebProviderTest(unittest.TestCase):
         web_site = self.getConfiguredWebSite(config)
 
         root_resource = self.getResourceForFakeRequest(web_site, [''])
-        self.assertConfiguredWithPipeline(root_resource, pipeline='pipeline.root_pipeline', missing_pipeline='pipeline.root_missing_pipeline')
+        self.assertConfiguredWithProcessor(root_resource, processor='pipeline.root_pipeline', no_resource_processor='pipeline.root_no_resource_pipeline')
 
-        # nonexistent resources should be rendered by the closest matching no-resource-pipeline
+        # nonexistent resources should be rendered by the closest matching no-resource-pipeline_dependency
         self.assertEquals(self.getResourceForFakeRequest(web_site, ['nonexistent']), root_resource)
         self.assertEquals(self.getResourceForFakeRequest(web_site, ['nonexistent', 'nested']), root_resource)
-        # since foo does not have a missing_pipeline, its no_resources should be rendered by the root_resource
+        # since foo does not have a no_resource_processor, its no_resources should be rendered by the root_resource
         self.assertEquals(self.getResourceForFakeRequest(web_site, ['foo', 'nonexistent']), root_resource)
         self.assertEquals(self.getResourceForFakeRequest(web_site, ['foo', 'nonexistent', 'nested']), root_resource)
-        # since bar does not have a pipeline/missing_pipeline, it should be rendered by the root_resource
+        # since bar does not have a processor/no_resource_processor, it should be rendered by the root_resource
         self.assertEquals(self.getResourceForFakeRequest(web_site, ['bar']), root_resource)
 
-        self.assertConfiguredWithPipeline(self.getResourceForFakeRequest(web_site, ['foo']), pipeline='pipeline.foo_pipeline')
-        self.assertConfiguredWithPipeline(self.getResourceForFakeRequest(web_site, ['foo', '']), pipeline='pipeline.foo_pipeline')
+        self.assertConfiguredWithProcessor(self.getResourceForFakeRequest(web_site, ['foo']), processor='pipeline.foo_pipeline')
+        self.assertConfiguredWithProcessor(self.getResourceForFakeRequest(web_site, ['foo', '']), processor='pipeline.foo_pipeline')
 
         baz_resource = self.getResourceForFakeRequest(web_site, ['bar', 'baz'])
-        self.assertConfiguredWithPipeline(baz_resource, missing_pipeline='pipeline.baz_pipeline')
+        self.assertConfiguredWithProcessor(baz_resource, no_resource_processor='pipeline.baz_pipeline')
 
-        # since baz has a missing_pipeline, it is capable of rendering that itself doesn't have a "proper" resource/pipeline
+        # since baz has a no_resource_processor, it is capable of rendering that itself doesn't have a "proper" resource/processor
         self.assertEquals(self.getResourceForFakeRequest(web_site, ['bar', 'baz', '']), baz_resource)
         self.assertEquals(self.getResourceForFakeRequest(web_site, ['bar', 'baz', 'nonexistent']), baz_resource)
         self.assertEquals(self.getResourceForFakeRequest(web_site, ['bar', 'baz', 'nonexistent', 'nested']), baz_resource)
 
-    def test_web_resource_missing_request_processing(self):
+    def test_web_resource_no_resource_request_processing(self):
         """ Test that various web resources are being rendered with a request instance that
         has its "postpath" instance variable set to the remaining / unhandled path segments.
         """
         config = dict(
             routing = dict(
-                __config__ = dict(pipeline='root_pipeline', missing_pipeline='root_missing_pipeline'),
+                __config__ = dict(processor='pipeline.root_pipeline', no_resource_processor='pipeline.root_no_resource_pipeline'),
                 foo = dict(
-                    __config__ = dict(pipeline = 'foo_pipeline')
+                    __config__ = dict(processor='pipeline.foo_pipeline')
                 ),
                 bar = dict(
                     baz = dict(
-                        __config__ = dict(missing_pipeline = 'baz_pipeline')
+                        __config__ = dict(no_resource_processor='pipeline.baz_pipeline')
                     )
                 )
             )
@@ -153,7 +174,6 @@ class WebProviderTest(unittest.TestCase):
         web_site = self.getConfiguredWebSite(config)
 
         batons = list()
-        pipeline = StubPipeline(batons.append)
 
         # fake the pipelines being ready:
         root_resource = self.getResourceForFakeRequest(web_site, [''])
@@ -161,10 +181,10 @@ class WebProviderTest(unittest.TestCase):
         baz_resource = self.getResourceForFakeRequest(web_site, ['bar', 'baz'])
 
         for resource in (root_resource, foo_resource, baz_resource):
-            if resource.pipeline_dependency:
-                resource.pipeline_dependency.on_resource_ready(pipeline)
-            if resource.missing_pipeline_dependency:
-                resource.missing_pipeline_dependency.on_resource_ready(pipeline)
+            if resource.processor_dependency:
+                resource.processor_dependency.on_resource_ready(batons.append)
+            if resource.no_resource_processor_dependency:
+                resource.no_resource_processor_dependency.on_resource_ready(batons.append)
 
         def assertRequestRenderedWithPostPath(web_site, batons, request, post_path):
             self.getResourceForFakeRequest(web_site, request=request).render(request)
@@ -173,29 +193,29 @@ class WebProviderTest(unittest.TestCase):
             self.assertEquals(request.postpath, post_path)
 
         for request_path, expected_postpath in (
-            # paths under the root resource, which has both a regular pipeline and a missing_pipeline
+            # paths under the root resource, which has both a regular processor and a no resource processor
             ([''], []),
             (['nonexistent'], ['nonexistent']),
             (['nonexistent', 'nested'], ['nonexistent', 'nested']),
 
-            # paths under the foo/bar resource, which only has a regular pipeline
+            # paths under the foo/bar resource, which only has a regular processor
             (['foo', 'bar'], ['foo', 'bar']),
             (['foo', 'bar', ''], ['foo', 'bar', '']),
             (['foo', 'bar', 'nested'], ['foo', 'bar', 'nested']),
 
-            # paths under the bar resource, which has a nested resource, but no pipelines at all
+            # paths under the bar resource, which has a nested resource, but no processors at all
             (['bar'], ['bar']),
             (['bar', ''], ['bar', '']),
             (['bar', 'nested'], ['bar', 'nested']),
 
-            # paths under the bar/baz resource, which only has a missing_pipeline
+            # paths under the bar/baz resource, which only has a no resource processor
             (['bar', 'baz'], []),
             (['bar', 'baz', ''], ['']),
             (['bar', 'baz', 'nested'], ['nested']),
             (['bar', 'baz', 'nested', ''], ['nested', '']),
             (['bar', 'baz', 'nested', 'deeply'], ['nested', 'deeply'])):
 
-            assertRequestRenderedWithPostPath(web_site, batons, web_provider.DummyRequest(request_path), expected_postpath)
+            assertRequestRenderedWithPostPath(web_site, batons, DummyRequest(request_path), expected_postpath)
 
     def test_static_preprocessors(self):
         current_file = filepath.FilePath(__file__)
@@ -216,23 +236,23 @@ class WebProviderTest(unittest.TestCase):
         web_site = self.getConfiguredWebSite(config)
 
         # send a request for this file:
-        request = web_provider.DummyRequest([current_file.basename()])
+        request = DummyRequest([current_file.basename()])
         resource = web_site.factory.getResourceFor(request)
         resource.render(request)
 
         self.assertEquals(request.responseHeaders.getRawHeaders('foo'), ['bar'])
 
-    def test_pipeline_routing_with_nested_resources(self):
+    def test_processor_routing_with_nested_resources(self):
         config = dict(
             routing = dict(
                 __config__ = dict(
-                    pipeline = 'a_pipeline',
+                    processor = 'pipeline.a_pipeline',
                     static = filepath.FilePath(__file__).dirname(),
                 ),
                 nested = dict(
                     deeply = dict(
                         __config__ = dict(
-                            pipeline = 'another_pipeline'
+                            processor = 'pipeline.another_pipeline'
                         )
                     )
                 )
@@ -243,7 +263,7 @@ class WebProviderTest(unittest.TestCase):
 
         web_resource = self.getResourceForFakeRequest(web_site, [''])
 
-        self.assertConfiguredWithPipeline(web_resource, 'pipeline.a_pipeline')
+        self.assertConfiguredWithProcessor(web_resource, 'pipeline.a_pipeline')
 
         # if we request an existing file, a static file resource will be returned
         filename = filepath.FilePath(__file__).basename()
@@ -251,37 +271,35 @@ class WebProviderTest(unittest.TestCase):
         self.assertIsInstance(static_resource, web_provider.StaticFile)
 
         web_resource = self.getResourceForFakeRequest(web_site, ['nested'])
-        self.assertConfiguredWithPipeline(web_resource)
+        self.assertConfiguredWithProcessor(web_resource)
 
-        missing = self.getResourceForFakeRequest(web_site, ['nested', 'nonexistent'])
-        self.assertIsInstance(missing, resource.NoResource)
+        no_resource = self.getResourceForFakeRequest(web_site, ['nested', 'nonexistent'])
+        self.assertIsInstance(no_resource, resource.NoResource)
 
         deeply_resource = self.getResourceForFakeRequest(web_site, ['nested', 'deeply'])
-        self.assertConfiguredWithPipeline(deeply_resource, 'pipeline.another_pipeline')
+        self.assertConfiguredWithProcessor(deeply_resource, 'pipeline.another_pipeline')
 
     def test_web_resource_simple_request_processing(self):
-        web_resource = self._create_configured_web_resource(dict(__config__=dict(pipeline='a_pipeline')))
+        web_resource = self._create_configured_web_resource(dict(__config__=dict(processor='pipeline.a_pipeline')))
 
-        request = web_provider.DummyRequest([''])
+        request = DummyRequest([''])
 
         batons = list()
-        pipeline = StubPipeline(batons.append)
-        web_resource.pipeline_dependency.on_resource_ready(pipeline)
+        web_resource.processor_dependency.on_resource_ready(batons.append)
 
-        # rendering the request should result in a baton being processed by the pipeline
+        # rendering the request should result in a baton being processed by the processor
         web_resource.render(request)
         self.assertEquals(batons, [dict(request=request)])
 
     def test_web_resource_processing_handles_exceptions(self):
-        web_resource = self._create_configured_web_resource(dict(__config__=dict(pipeline='a_pipeline')))
+        web_resource = self._create_configured_web_resource(dict(__config__=dict(processor='pipeline.a_pipeline')))
 
-        request = web_provider.DummyRequest([''])
+        request = DummyRequest([''])
 
         def raiser(baton):
             raise Exception()
 
-        pipeline = StubPipeline(raiser)
-        web_resource.pipeline_dependency.on_resource_ready(pipeline)
+        web_resource.processor_dependency.on_resource_ready(raiser)
 
         # rendering the request should result in an exception response
         web_resource.render(request)
@@ -290,18 +308,17 @@ class WebProviderTest(unittest.TestCase):
         self.assertEquals(request.code, 500)
 
     def test_web_resource_processing_raises_with_debugging(self):
-        routing = dict(__config__=dict(pipeline='a_pipeline'))
+        routing = dict(__config__=dict(processor='pipeline.a_pipeline'))
         site_config = dict(debug=dict(allow=['localhost']))
         web_resource = self._create_configured_web_resource(routing, site_config)
 
-        request = web_provider.DummyRequest([''])
+        request = DummyRequest([''])
         request.client = address.IPv4Address('TCP', 'localhost', 1234)
 
         def raiser(baton):
             raise Exception()
 
-        pipeline = StubPipeline(raiser)
-        web_resource.pipeline_dependency.on_resource_ready(pipeline)
+        web_resource.processor_dependency.on_resource_ready(raiser)
 
         # rendering the request should result in an exception response
         web_resource.render(request)
@@ -337,7 +354,7 @@ class WebProviderTest(unittest.TestCase):
         f = failure.Failure(Exception())
         path = debug_handler.register_failure(f)
 
-        request = web_provider.DummyRequest([path])
+        request = DummyRequest([path])
 
         # localhost is not allowed to debug:
         request.client = address.IPv4Address('TCP', 'localhost', 1234)
@@ -355,11 +372,11 @@ class WebProviderTest(unittest.TestCase):
         try:
             raise Exception()
         except Exception as e:
-            f = failure.Failure()
+            f = util.NonCleaningFailure()
 
         web_debugger = web_provider.WebDebugger(f)
 
-        request = web_provider.DummyRequest([])
+        request = DummyRequest([])
         request.addArg('expr', 'foo')
         result = web_debugger.render(request)
 
@@ -373,28 +390,26 @@ class WebProviderTest(unittest.TestCase):
             self.assertRaises(exceptions.ConfigurationError, site.configure, self.runtime_environment)
 
     def test_request_finished_when_garbage_collected(self):
-        web_site = web_provider.WebSite('site_name', dict(routing=dict(__config__=dict(pipeline='test_pipeline'))))
+        web_site = web_provider.WebSite('site_name', dict(routing=dict(__config__=dict(processor='pipeline.test_pipeline'))))
         web_site.configure(self.runtime_environment)
 
-        # create a stub pipeline that collects the batons
         batons = list()
-        stub_pipeline = StubPipeline(batons.append)
 
         web_resource = self.getResourceForFakeRequest(web_site, [])
-        web_resource.pipeline_dependency = dependencies.InstanceDependency(stub_pipeline)
-        web_resource.pipeline_dependency.is_ready = True
+        web_resource.processor_dependency = dependencies.InstanceDependency(batons.append)
+        web_resource.processor_dependency.is_ready = True
 
-        request = web_provider.DummyRequest([])
+        request = DummyRequest([])
         web_resource.render(request)
 
-        # the pipeline should have been asked to process a baton
+        # the processor should have been asked to process a baton
         self.assertEquals(len(batons), 1)
         self.assertEquals(batons[0]['request'], request)
 
-        # the pipeline didn't finish the request:
+        # the processor didn't finish the request:
         self.assertEquals(request.finished, False)
 
-        # .. however, when the pipeline loses the reference to the request, it should be
+        # .. however, when the processor loses the reference to the request, it should be
         # automatically finished:
         batons.pop()
         self.assertEquals(request.finished, True)
@@ -407,7 +422,7 @@ class TestConcatenatedFile(unittest.TestCase):
         file_paths = [test_data_path.child('foo'), test_data_path.child('bar')]
         cf = web_provider.ConcatenatedFile('text/plain', file_paths)
 
-        request = web_provider.DummyRequest([''])
+        request = DummyRequest([''])
         text = cf.render_GET(request)
 
         self.assertEquals(text, 'foo\nbar\n')
@@ -417,7 +432,7 @@ class TestConcatenatedFile(unittest.TestCase):
         file_paths = [test_data_path.child('bar'), test_data_path.child('foo')]
         cf = web_provider.ConcatenatedFile('text/plain', file_paths)
 
-        request = web_provider.DummyRequest([''])
+        request = DummyRequest([''])
         text = cf.render_GET(request)
 
         self.assertEquals(text, 'bar\nfoo\n')
@@ -427,7 +442,7 @@ class TestConcatenatedFile(unittest.TestCase):
         file_paths = [test_data_path.child('foo')]
         cf = web_provider.ConcatenatedFile('text/plain', file_paths)
 
-        request = web_provider.DummyRequest([''])
+        request = DummyRequest([''])
         text = cf.render_GET(request)
 
         self.assertEquals(text, 'foo\n')
@@ -436,7 +451,7 @@ class TestConcatenatedFile(unittest.TestCase):
         file_paths = []
         cf = web_provider.ConcatenatedFile('text/plain', file_paths)
 
-        request = web_provider.DummyRequest([''])
+        request = DummyRequest([''])
         text = cf.render_GET(request)
 
         self.assertEquals(text, '')
@@ -445,7 +460,7 @@ class TestConcatenatedFile(unittest.TestCase):
         file_paths = []
         cf = web_provider.ConcatenatedFile('text/plain', file_paths)
 
-        request = web_provider.DummyRequest([''])
+        request = DummyRequest([''])
         cf.render_GET(request)
 
         self.assertEquals(request.responseHeaders.getRawHeaders('content-type'), ['text/plain'])
